@@ -1,4 +1,4 @@
-/* $Id: ruby_xml_node.h,v 1.3 2006/11/20 01:22:07 roscopeco Exp $ */
+/* $Id: ruby_xml_node.h 189 2007-09-26 15:04:47Z danj $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
@@ -12,18 +12,16 @@ extern VALUE eXMLNodeUnknownType;
 
 typedef struct ruby_xml_node {
   xmlNodePtr node;
-  VALUE xd;
-  //int is_ptr;
 } ruby_xml_node;
 
+VALUE
+ruby_xml_node2_wrap(VALUE class, xmlNodePtr xnode);
+
 void ruby_xml_node_free(ruby_xml_node *rxn);
+void ruby_xml_node_mark_common(xmlNodePtr n);
 void ruby_init_xml_node(void);
 VALUE ruby_xml_node_child_set(VALUE self, VALUE obj);
-VALUE ruby_xml_node_new(VALUE class, xmlNodePtr node);
-VALUE ruby_xml_node_new_ptr(VALUE class, VALUE xd, xmlNodePtr node);
-VALUE ruby_xml_node_new3(VALUE class, VALUE xd, xmlNodePtr node, int ptr);
 VALUE ruby_xml_node_name_get(VALUE self);
 VALUE ruby_xml_node_property_get(VALUE self, VALUE key);
 VALUE ruby_xml_node_property_set(VALUE self, VALUE key, VALUE val);
-VALUE ruby_xml_node_set_ptr(VALUE node, int is_ptr);
 #endif
