@@ -1,4 +1,4 @@
-# $Id: tc_xml_document.rb,v 1.3 2006/04/17 13:30:20 roscopeco Exp $
+# $Id: tc_xml_document.rb 183 2007-09-21 14:09:52Z danj $
 require 'test/unit'
 require "libxml_test"
 
@@ -18,9 +18,10 @@ class TC_XML_Document < Test::Unit::TestCase
 
   def test_libxml_document_find()
     set = @doc.find('/ruby_array/fixnum')
-    assert_instance_of(XML::Node::Set, set)
-    xpt = set.xpath
-    assert_instance_of(XML::XPath, xpt)
+    assert_instance_of(XML::XPath::Object, set)
+    assert_raise(NoMethodError) {
+      xpt = set.xpath
+    }
   end
 
   def test_ruby_xml_document_compression()
