@@ -1,4 +1,4 @@
-# $Id: tc_xml_node.rb,v 1.3 2006/04/17 13:30:19 roscopeco Exp $
+# $Id: tc_xml_node.rb 191 2007-10-05 14:56:49Z danj $
 require "libxml_test"
 require 'test/unit'
 
@@ -12,9 +12,10 @@ class TC_XML_Node < Test::Unit::TestCase
     @root = doc.root
     assert_instance_of(XML::Node, @root)
     set = doc.find('/ruby_array/fixnum')
-    assert_instance_of(XML::Node::Set, set)
-    xpt = set.xpath
-    assert_instance_of(XML::XPath, xpt)
+    assert_instance_of(XML::XPath::Object, set)
+    assert_raise(NoMethodError) {
+      xpt = set.xpath
+    }
     @nodes = []
     set.each do |n|
       @nodes.push(n)
@@ -50,7 +51,7 @@ class TC_XML_Node < Test::Unit::TestCase
   end
 
   def test_libxml_node_find()
-    set = @root.find('./fixnum')
+    set = @root.find('./fixnum').set
     assert_instance_of(XML::Node::Set, set)
     for node in set
       assert_instance_of(XML::Node, node)
