@@ -1,4 +1,4 @@
-# $Id: tc_xml_node_set.rb,v 1.3 2006/04/17 13:30:19 roscopeco Exp $
+# $Id: tc_xml_node_set.rb 183 2007-09-21 14:09:52Z danj $
 require "libxml_test"
 require 'test/unit'
 
@@ -9,7 +9,7 @@ class TC_XML_Node_Set < Test::Unit::TestCase
     assert_equal(str, xp.string = str)
     doc = xp.parse
     assert_instance_of(XML::Document, doc)
-    @set = doc.find('/ruby_array/fixnum')
+    @set = doc.find('/ruby_array/fixnum').set
     assert_instance_of(XML::Node::Set, @set)    
   end
 
