@@ -1,4 +1,4 @@
-# $Id: tc_xml_node_set2.rb,v 1.3 2006/11/20 01:22:08 roscopeco Exp $
+# $Id: tc_xml_node_set2.rb 183 2007-09-21 14:09:52Z danj $
 require "libxml_test"
 require 'test/unit'
 
@@ -11,8 +11,8 @@ class TC_XML_Node_Set2 < Test::Unit::TestCase
     assert_instance_of(XML::Document, doc)
     @one = doc.root.child
     @two = @one.next    
-    @set = doc.find('/ruby_array/fixnum')
-    @emptyset = doc.find('/fixnum')
+    @set = doc.find('/ruby_array/fixnum').set
+    @emptyset = doc.find('/fixnum').set
     assert_instance_of(XML::Node::Set, @set)    
   end
 
