@@ -1,4 +1,4 @@
-# $Id: tc_xml_xpath.rb,v 1.3 2006/04/17 13:30:19 roscopeco Exp $
+# $Id: tc_xml_xpath.rb 183 2007-09-21 14:09:52Z danj $
 require "libxml_test"
 require "test/unit"
 
@@ -9,8 +9,8 @@ class TC_XML_XPath < Test::Unit::TestCase
     assert_equal(str, xp.string = str)
     doc = xp.parse
     assert_instance_of(XML::Document, doc)
-    @xpt = doc.find('/ruby_array/fixnum').xpath
-    assert_instance_of(XML::XPath, @xpt)
+    @xpt = doc.find('/ruby_array/fixnum')
+    assert_instance_of(XML::XPath::Object, @xpt)
   end
 
   def teardown()
